# Penrose tiling
#
# rewrote Python code in http://preshing.com/20110831/penrose-tiling-explained
# for Ruby using TurtleField
# Y. Hayakawa, 2013

require 'complex'
require './turtle.rb'

GOLDENRATIO = (1 + Math.sqrt(5)) / 2

class PenroseTile
  @triangles
  @turtle
  def initialize(t)
    @turtle = t
    @triangles = []
  end

  def subdivide(triangles)
    result = []
    for color,a,b,c in triangles do
      if color == 0 then
        p = a + (b - a) / GOLDENRATIO
        result << [0, c, p, b]
        result << [1, p, c, a]
      else
        q = b + (a - b) / GOLDENRATIO
        r = b + (c - b) / GOLDENRATIO
        result << [1, r, c, a]
        result << [1, q, r, b]
        result << [0, r, q, a]
      end
    end
    return result
  end

  def gen(level)
    @triangles=[ ]
    for i in 0..9 do
      b = Complex.polar(1, (2*i - 1) * Math::PI / 10)
      c = Complex.polar(1, (2*i + 1) * Math::PI / 10)
      if i % 2 == 0 then
        b,c = c,b  
      end
      @triangles << [0, 0*Complex::I, b, c]
    end
    level.times do
      @triangles = subdivide(@triangles)
    end
  end

  def draw(mag)
    @turtle.clr
    for color,a,b,c in @triangles do
      @turtle.fill
      if color == 0 then
        @turtle.col(1.0,0.3,0.0)
      else
        @turtle.col(0.3,0.2,1.0)
      end
      @turtle.jump(a.real*mag, a.imag*mag) 
      @turtle.pd
      @turtle.jump(b.real*mag, b.imag*mag) 
      @turtle.jump(c.real*mag, c.imag*mag) 
      @turtle.pu

      @turtle.col(0.0,0.0,0.0)
      @turtle.line

      @turtle.jump(b.real*mag, b.imag*mag) 
      @turtle.pd
      @turtle.jump(a.real*mag, a.imag*mag) 
      @turtle.jump(c.real*mag, c.imag*mag) 
      @turtle.pu
    end    
  end
end

# main part

ttl = Turtle.new
tile = PenroseTile.new(ttl)
tile.gen(5)
tile.draw(250)
